<?php

namespace Drupal\media_name;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\field\FieldConfigInterface;
use Drupal\media\MediaInterface;

/**
 * Class MediaName.
 */
class MediaName implements MediaNameInterface {

  /**
   * Drupal\Core\Entity\EntityFieldManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * Drupal\Core\Messenger\MessengerInterface definition.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Drupal\Core\Logger\LoggerChannelFactoryInterface definition.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructs a new MediaName object.
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager, MessengerInterface $messenger, LoggerChannelFactoryInterface $logger_factory) {
    $this->entityFieldManager = $entity_field_manager;
    $this->messenger = $messenger;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function alterMediaEditForm(FormStateInterface $form_state) {
    // Get the media bundle file field.
    $media = $this->getFormMedia($form_state);
    // On create, skip.
    // The default behaviour works as expected for this module use case:
    // if the name is empty, use the file name, otherwise preserve
    // the name override.
    if ($media->isNew()) {
      return;
    }

    $fileName = $this->getMediaFileName($media);
    if (!is_null($fileName) && $fileName !== $media->label()) {
      $form_state->addBuildInfo(self::MEDIA_NAME_ORIGINAL_VALUE, $media->label());
      $form_state->addBuildInfo(self::FILENAME_ORIGINAL_VALUE, $fileName);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function mediaEditFormSubmitHandler(FormStateInterface $form_state) {
    $buildInfo = $form_state->getBuildInfo();
    $media = $this->getFormMedia($form_state);
    $fileName = $this->getMediaFileName($media);
    if (empty($buildInfo[self::MEDIA_NAME_ORIGINAL_VALUE])
        || empty($buildInfo[self::FILENAME_ORIGINAL_VALUE])
        || empty($form_state->getValue('name')[0]['value'])
    ) {
      return;
    }

    // If the file name has changed, it will override the Media name.
    // In this case only, we need to set the original Media name back.
    if ($fileName !== $buildInfo[self::FILENAME_ORIGINAL_VALUE]) {
      $media->setName($form_state->getValue('name')[0]['value']);
      try {
        $media->save();
      }
      catch (\Exception $exception) {
        $this->loggerFactory->get('media_name')->error($exception->getMessage());
        $this->messenger->addError($exception->getMessage());
      }
    }
  }

  /**
   * Returns the Media entity from a Form.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return \Drupal\media\MediaInterface
   *   Media entity.
   */
  private function getFormMedia(FormStateInterface $form_state) {
    $formObject = $form_state->getFormObject();
    return $formObject->getEntity();
  }

  /**
   * Returns the File name from the File field of a Media entity.
   *
   * @return string|null
   *   File name if it is found or NULL.
   */
  public function getMediaFileName(MediaInterface $media) {
    $result = NULL;
    $fields = $this->entityFieldManager->getFieldDefinitions('media', $media->bundle());
    foreach ($fields as $key => $field) {
      // Only get file related fields, exclude other fields
      // and then Media bundles that are not containing files
      // (e.g. Remote video).
      if ($field instanceof FieldConfigInterface
        && in_array($field->getType(), ['file', 'image'])
        && $media->hasField($field->getName())
        && !$media->get($field->getName())->isEmpty()
      ) {
        /*** @var \Drupal\file\FileInterface $file */
        $file = $media->get($field->getName())->referencedEntities()[0];
        $result = $file->getFilename();
      }
    }
    return $result;
  }

}
